"""Linear Estimators."""
#  Copyright 2015-present Scikit Flow Authors. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from __future__ import division, print_function, absolute_import

from sklearn.base import ClassifierMixin, RegressorMixin

from skflow.estimators.base import TensorFlowEstimator
from skflow import models


class TensorFlowLinearRegressor(TensorFlowEstimator, RegressorMixin):
    """TensorFlow Linear Regression model."""

    def __init__(self, n_classes=0, tf_master="", batch_size=32, steps=200, optimizer="SGD",
                 learning_rate=0.1, tf_random_seed=42, continue_training=False,
                 num_cores=4, verbose=1, early_stopping_rounds=None,
                 max_to_keep=5, keep_checkpoint_every_n_hours=10000):
        super(TensorFlowLinearRegressor, self).__init__(
            model_fn=models.linear_regression, n_classes=n_classes,
            tf_master=tf_master,
            batch_size=batch_size, steps=steps, optimizer=optimizer,
            learning_rate=learning_rate, tf_random_seed=tf_random_seed,
            continue_training=continue_training, num_cores=num_cores,
            verbose=verbose, early_stopping_rounds=early_stopping_rounds,
            max_to_keep=max_to_keep,
            keep_checkpoint_every_n_hours=keep_checkpoint_every_n_hours)

    @property
    def weights_(self):
        """Returns weights of the linear regression."""
        return self.get_tensor_value('linear_regression/weights:0')

    @property
    def bias_(self):
        """Returns bias of the linear regression."""
        return self.get_tensor_value('linear_regression/bias:0')


class TensorFlowLinearClassifier(TensorFlowEstimator, ClassifierMixin):
    """TensorFlow Linear Classifier model."""

    def __init__(self, n_classes, tf_master="", batch_size=32, steps=200, optimizer="SGD",
                 learning_rate=0.1, class_weight=None,
                 tf_random_seed=42, continue_training=False, num_cores=4,
                 verbose=1, early_stopping_rounds=None,
                 max_to_keep=5, keep_checkpoint_every_n_hours=10000):
        super(TensorFlowLinearClassifier, self).__init__(
            model_fn=models.logistic_regression, n_classes=n_classes,
            tf_master=tf_master,
            batch_size=batch_size, steps=steps, optimizer=optimizer,
            learning_rate=learning_rate, class_weight=class_weight,
            tf_random_seed=tf_random_seed,
            continue_training=continue_training, num_cores=num_cores,
            verbose=verbose, early_stopping_rounds=early_stopping_rounds,
            max_to_keep=max_to_keep,
            keep_checkpoint_every_n_hours=keep_checkpoint_every_n_hours)

    @property
    def weights_(self):
        """Returns weights of the linear classifier."""
        return self.get_tensor_value('logistic_regression/weights:0')

    @property
    def bias_(self):
        """Returns weights of the linear classifier."""
        return self.get_tensor_value('logistic_regression/bias:0')


TensorFlowRegressor = TensorFlowLinearRegressor
TensorFlowClassifier = TensorFlowLinearClassifier



# encoding: utf-8

#  Copyright 2015-present Scikit Flow Authors. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import numpy as np
import pandas
import tensorflow as tf

from skflow.preprocessing import categorical


class CategoricalTest(tf.test.TestCase):

    def testSingleCategoricalProcessor(self):
        cat_processor = categorical.CategoricalProcessor(
            min_frequency=1)
        X = cat_processor.fit_transform(
            [["0"], [1], [float('nan')], 
            ["C"], ["C"], [1], ["0"], [np.nan], [3]])
        self.assertAllEqual(list(X), [
            [1], [2], [0], [3], 
            [3], [2], [1], [0],
            [0]])

    def testSingleCategoricalProcessorPandasSingleDF(self):
        cat_processor = categorical.CategoricalProcessor()
        data = pandas.DataFrame({"Gender": ["Male", "Female", "Male"]})
        X = list(cat_processor.fit_transform(data))
        self.assertAllEqual(list(X), [[1], [2], [1]]) 

    def testMultiCategoricalProcessor(self):
        cat_processor = categorical.CategoricalProcessor(
            min_frequency=0, share=False)
        x = cat_processor.fit_transform(
            [["0", "Male"], [1, "Female"], ["3", "Male"]])
        self.assertAllEqual(list(x), [[1, 1], [2, 2], [3, 1]])


if __name__ == "__main__":
    tf.test.main()



#  Copyright 2015-present Scikit Flow Authors. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import numpy as np
import tensorflow as tf

from skflow import ops


class DropoutTest(tf.test.TestCase):

    def test_dropout_float(self):
        with self.test_session() as session:
            x = tf.placeholder(tf.float32, [5, 5])
            y = ops.dropout(x, 0.5)
            probs = tf.get_collection(ops.DROPOUTS)
            session.run(tf.initialize_all_variables())
            self.assertEqual(len(probs), 1)
            self.assertEqual(session.run(probs[0]), 0.5)

    def test_dropout_tensor(self):
        with self.test_session():
            x = tf.placeholder(tf.float32, [5, 5])
            y = tf.get_variable("prob", [], initializer=tf.constant_initializer(0.5))
            z = ops.dropout(x, y)
            probs = tf.get_collection(ops.DROPOUTS)
            self.assertEqual(probs, [y])


if __name__ == '__main__':
    tf.test.main()
